# Copyright 2015 Hitachi Data Systems.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.


import ddt
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc
import testtools
from testtools import testcase as tc

from manila_tempest_tests.common import constants
from manila_tempest_tests.common import waiters
from manila_tempest_tests import share_exceptions
from manila_tempest_tests.tests.api import base
from manila_tempest_tests import utils

CONF = config.CONF


@ddt.ddt
class MigrationNegativeTest(base.BaseSharesAdminTest):
    """Tests Share Migration.

    Tests share migration in multi-backend environment.
    """

    protocol = "nfs"

    @classmethod
    def resource_setup(cls):
        super(MigrationNegativeTest, cls).resource_setup()
        if cls.protocol not in CONF.share.enable_protocols:
            message = "%s tests are disabled." % cls.protocol
            raise cls.skipException(message)
        if not (CONF.share.run_host_assisted_migration_tests or
                CONF.share.run_driver_assisted_migration_tests):
            raise cls.skipException("Share migration tests are disabled.")

        pools = cls.shares_client.list_pools(detail=True)['pools']

        if len(pools) < 2:
            raise cls.skipException("At least two different pool entries "
                                    "are needed to run share migration tests.")

        # create share type (generic)
        extra_specs = {}

        if CONF.share.capability_snapshot_support:
            extra_specs.update({'snapshot_support': True})
        cls.share_type = cls.create_share_type(extra_specs=extra_specs)
        cls.share_type_id = cls.share_type['id']

        # create share
        cls.share = cls.create_share(cls.protocol,
                                     size=CONF.share.share_size + 1,
                                     share_type_id=cls.share_type_id)
        cls.share = cls.shares_client.get_share(cls.share['id'])['share']

        dest_pool = utils.choose_matching_backend(
            cls.share, pools, cls.share_type)

        if not dest_pool or dest_pool.get('name') is None:
            raise share_exceptions.ShareMigrationException(
                "No valid pool entries to run share migration tests.")

        cls.dest_pool = dest_pool['name']

        cls.new_type_invalid = cls.create_share_type(
            name=data_utils.rand_name(
                'new_invalid_share_type_for_migration'),
            cleanup_in_class=True,
            extra_specs=utils.get_configured_extra_specs(variation='invalid'))

    @decorators.idempotent_id('8aa1f2a0-bc44-4df5-a556-161590e594a3')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    @utils.skip_if_microversion_not_supported("2.22")
    def test_migration_cancel_invalid(self):
        self.assertRaises(
            lib_exc.BadRequest, self.shares_v2_client.migration_cancel,
            self.share['id'])

    @decorators.idempotent_id('6d0dfb2e-51a0-4cb7-8c69-6135a49c6057')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    @utils.skip_if_microversion_not_supported("2.22")
    def test_migration_get_progress_None(self):
        self.shares_v2_client.reset_task_state(self.share["id"], None)
        waiters.wait_for_resource_status(
            self.shares_v2_client, self.share["id"], None,
            status_attr='task_state')
        self.assertRaises(
            lib_exc.BadRequest, self.shares_v2_client.migration_get_progress,
            self.share['id'])

    @decorators.idempotent_id('2ab1fc82-bc13-4c99-8324-c6b23530e8a4')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    @utils.skip_if_microversion_not_supported("2.22")
    def test_migration_complete_invalid(self):
        self.assertRaises(
            lib_exc.BadRequest, self.shares_v2_client.migration_complete,
            self.share['id'])

    @decorators.idempotent_id('8ef562b4-7704-4a78-973f-9bf8d2b6f6a6')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    @utils.skip_if_microversion_not_supported("2.22")
    def test_migration_cancel_not_found(self):
        self.assertRaises(
            lib_exc.NotFound, self.shares_v2_client.migration_cancel,
            'invalid_share_id')

    @decorators.idempotent_id('044c792b-63e0-42c3-9f44-dc2280e2af08')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    @utils.skip_if_microversion_not_supported("2.22")
    def test_migration_get_progress_not_found(self):
        self.assertRaises(
            lib_exc.NotFound, self.shares_v2_client.migration_get_progress,
            'invalid_share_id')

    @decorators.idempotent_id('a509871a-3f3a-4618-bb60-9661732dd371')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    @utils.skip_if_microversion_not_supported("2.22")
    def test_migration_complete_not_found(self):
        self.assertRaises(
            lib_exc.NotFound, self.shares_v2_client.migration_complete,
            'invalid_share_id')

    @decorators.idempotent_id('6276bea6-6939-4569-930f-218d99c0fa56')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    @utils.skip_if_microversion_not_supported("2.29")
    @testtools.skipUnless(CONF.share.run_snapshot_tests,
                          "Snapshot tests are disabled.")
    def test_migrate_share_with_snapshot(self):
        snap = self.create_snapshot_wait_for_active(self.share['id'])
        self.assertRaises(
            lib_exc.Conflict, self.shares_v2_client.migrate_share,
            self.share['id'], self.dest_pool,
            force_host_assisted_migration=True)
        self.shares_v2_client.delete_snapshot(snap['id'])
        self.shares_v2_client.wait_for_resource_deletion(snapshot_id=snap[
            "id"])

    @decorators.idempotent_id('78670c24-c4ee-45b5-b166-2d053c333144')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    @utils.skip_if_microversion_not_supported("2.29")
    @ddt.data(True, False)
    def test_migrate_share_same_host(self, specified):
        new_share_type_id = None
        new_share_network_id = None
        if specified:
            new_share_type_id = self.share_type_id
            new_share_network_id = self.share['share_network_id']
        self.migrate_share(
            self.share['id'], self.share['host'],
            wait_for_status=constants.TASK_STATE_MIGRATION_SUCCESS,
            new_share_type_id=new_share_type_id,
            new_share_network_id=new_share_network_id)
        # NOTE(ganso): No need to assert, it is already waiting for correct
        # status (migration_success).

    @decorators.idempotent_id('af17204f-ffab-4ba8-8cb6-032e49216f67')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    @utils.skip_if_microversion_not_supported("2.29")
    def test_migrate_share_host_invalid(self):
        self.assertRaises(
            lib_exc.NotFound, self.shares_v2_client.migrate_share,
            self.share['id'], 'invalid_host')

    @decorators.idempotent_id('0558e9c4-0416-41d2-b28a-803d4b81521a')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    @utils.skip_if_microversion_not_supported("2.29")
    @ddt.data({'writable': False, 'preserve_metadata': False,
               'preserve_snapshots': False, 'nondisruptive': True},
              {'writable': False, 'preserve_metadata': False,
               'preserve_snapshots': True, 'nondisruptive': False},
              {'writable': False, 'preserve_metadata': True,
               'preserve_snapshots': False, 'nondisruptive': False},
              {'writable': True, 'preserve_metadata': False,
               'preserve_snapshots': False, 'nondisruptive': False})
    @ddt.unpack
    def test_migrate_share_host_assisted_not_allowed_API(
            self, writable, preserve_metadata, preserve_snapshots,
            nondisruptive):
        self.assertRaises(
            lib_exc.BadRequest, self.shares_v2_client.migrate_share,
            self.share['id'], self.dest_pool,
            force_host_assisted_migration=True, writable=writable,
            preserve_metadata=preserve_metadata, nondisruptive=nondisruptive,
            preserve_snapshots=preserve_snapshots)

    @decorators.idempotent_id('ee57024c-d00e-4def-8eec-cbc62bae327f')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    @utils.skip_if_microversion_not_supported("2.29")
    def test_migrate_share_change_type_no_valid_host(self):
        if not CONF.share.multitenancy_enabled:
            new_share_network_id = self.create_share_network(
                neutron_net_id='fake_net_id',
                neutron_subnet_id='fake_subnet_id')['id']
        else:
            new_share_network_id = None

        self.shares_v2_client.migrate_share(
            self.share['id'], self.dest_pool,
            new_share_type_id=self.new_type_invalid['id'],
            new_share_network_id=new_share_network_id)
        waiters.wait_for_migration_status(
            self.shares_v2_client, self.share['id'], self.dest_pool,
            constants.TASK_STATE_MIGRATION_ERROR)

    @decorators.idempotent_id('e2bd0cca-c091-4785-a9dc-7f42d2bb95a5')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    @utils.skip_if_microversion_not_supported("2.29")
    def test_migrate_share_not_found(self):
        self.assertRaises(
            lib_exc.NotFound, self.shares_v2_client.migrate_share,
            'invalid_share_id', self.dest_pool)

    @decorators.idempotent_id('86b427a7-27c0-4cd5-8f52-9688b339980b')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    @utils.skip_if_microversion_not_supported("2.29")
    def test_migrate_share_not_available(self):
        self.shares_client.reset_state(self.share['id'],
                                       constants.STATUS_ERROR)
        waiters.wait_for_resource_status(
            self.shares_v2_client, self.share['id'], constants.STATUS_ERROR)
        self.assertRaises(
            lib_exc.BadRequest, self.shares_v2_client.migrate_share,
            self.share['id'], self.dest_pool)
        self.shares_client.reset_state(self.share['id'],
                                       constants.STATUS_AVAILABLE)
        waiters.wait_for_resource_status(
            self.shares_v2_client, self.share['id'],
            constants.STATUS_AVAILABLE)

    @decorators.idempotent_id('e8f1e491-697a-4941-bf51-4d37f0a93fa5')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    @utils.skip_if_microversion_not_supported("2.29")
    def test_migrate_share_invalid_share_network(self):
        self.assertRaises(
            lib_exc.BadRequest, self.shares_v2_client.migrate_share,
            self.share['id'], self.dest_pool,
            new_share_network_id='invalid_net_id')

    @decorators.idempotent_id('be262d44-2ca2-4b9c-be3a-5a6a98ed871b')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    @utils.skip_if_microversion_not_supported("2.29")
    def test_migrate_share_invalid_share_type(self):
        self.assertRaises(
            lib_exc.BadRequest, self.shares_v2_client.migrate_share,
            self.share['id'], self.dest_pool,
            new_share_type_id='invalid_type_id')

    @decorators.idempotent_id('16c72693-6f9e-4cb4-a166-c60accd3479b')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    @utils.skip_if_microversion_not_supported("2.29")
    def test_migrate_share_opposite_type_share_network_invalid(self):

        extra_specs = utils.get_configured_extra_specs(
            variation='opposite_driver_modes')

        new_type_opposite = self.create_share_type(
            name=data_utils.rand_name('share_type_migration_negative'),
            extra_specs=extra_specs)

        new_share_network_id = None

        if CONF.share.multitenancy_enabled:

            new_share_network_id = self.create_share_network(
                neutron_net_id='fake_net_id',
                neutron_subnet_id='fake_subnet_id')['id']

        self.assertRaises(
            lib_exc.BadRequest, self.shares_v2_client.migrate_share,
            self.share['id'], self.dest_pool,
            new_share_type_id=new_type_opposite['id'],
            new_share_network_id=new_share_network_id)

    @decorators.idempotent_id('1f529b09-e404-4f0e-9423-bb4b117b5522')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    @utils.skip_if_microversion_not_supported("2.48")
    def test_share_type_azs_share_migrate_unsupported_az(self):
        extra_specs = self.add_extra_specs_to_dict({
            'availability_zones': 'non-existent az'})
        new_share_type = self.create_share_type(
            name=data_utils.rand_name('share_type_specific_az'),
            extra_specs=extra_specs, cleanup_in_class=False)
        self.assertRaises(
            lib_exc.BadRequest, self.shares_v2_client.migrate_share,
            self.share['id'], self.dest_pool,
            new_share_type_id=new_share_type['id'])

    @decorators.idempotent_id('90cf0ae4-4251-4142-bfa8-41f67a9e5b23')
    @testtools.skipUnless(CONF.share.run_driver_assisted_migration_tests,
                          "Driver-assisted migration tests are disabled.")
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    @utils.skip_if_microversion_not_supported("2.29")
    def test_create_snapshot_during_share_migration(self):
        self._test_share_actions_during_share_migration('create_snapshot', [])

    @decorators.idempotent_id('20121039-bb11-45d8-9972-d2daff7a779c')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    @utils.skip_if_microversion_not_supported("2.29")
    @ddt.data(('extend_share', [CONF.share.share_size + 2]),
              ('shrink_share', [CONF.share.share_size]))
    @ddt.unpack
    def test_share_resize_during_share_migration(self, method_name, *args):
        self._test_share_actions_during_share_migration(method_name, *args)

    def skip_if_tests_are_disabled(self, method_name):
        property_to_evaluate = {
            'extend_share': CONF.share.run_extend_tests,
            'shrink_share': CONF.share.run_shrink_tests,
            'create_snapshot': CONF.share.run_snapshot_tests,
        }
        if not property_to_evaluate[method_name]:
            raise self.skipException(method_name + 'tests are disabled.')

    @decorators.idempotent_id('6e83fc25-4e3e-49a7-93e8-db4e6b355a91')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    @utils.skip_if_microversion_not_supported("2.29")
    def test_add_access_rule_during_migration(self):
        access_type = "ip"
        access_to = "50.50.50.50"
        self.shares_v2_client.reset_state(self.share['id'],
                                          constants.STATUS_MIGRATING)
        self.shares_v2_client.reset_task_state(
            self.share['id'],
            constants.TASK_STATE_MIGRATION_DRIVER_PHASE1_DONE)
        self.assertRaises(
            lib_exc.BadRequest,
            self.shares_v2_client.create_access_rule,
            self.share['id'], access_type, access_to)
        # Revert the migration state by cancelling the migration
        self.shares_v2_client.reset_state(self.share['id'],
                                          constants.STATUS_AVAILABLE)
        self.shares_v2_client.reset_task_state(
            self.share['id'],
            constants.TASK_STATE_MIGRATION_CANCELLED)

    def _test_share_actions_during_share_migration(self, method_name, *args):
        self.skip_if_tests_are_disabled(method_name)
        # Verify various share operations during share migration
        self.shares_v2_client.reset_state(self.share['id'],
                                          constants.STATUS_MIGRATING)
        self.shares_v2_client.reset_task_state(
            self.share['id'],
            constants.TASK_STATE_MIGRATION_DRIVER_PHASE1_DONE)

        self.assertRaises(
            lib_exc.BadRequest, getattr(self.shares_v2_client, method_name),
            self.share['id'], *args)
        # Revert the migration state by cancelling the migration
        self.shares_v2_client.reset_state(self.share['id'],
                                          constants.STATUS_AVAILABLE)
        self.shares_v2_client.reset_task_state(
            self.share['id'],
            constants.TASK_STATE_MIGRATION_CANCELLED)



# Copyright 2015 Mirantis Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import ddt
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc
import testtools
from testtools import testcase as tc

from manila_tempest_tests.common import waiters
from manila_tempest_tests.tests.api import base
from manila_tempest_tests import utils

CONF = config.CONF


@ddt.ddt
class SharesActionsNegativeTest(base.BaseSharesMixedTest):

    @classmethod
    def resource_setup(cls):
        super(SharesActionsNegativeTest, cls).resource_setup()
        cls.admin_client = cls.admin_shares_v2_client
        cls.share_name = data_utils.rand_name("tempest-share-name")
        cls.share_desc = data_utils.rand_name("tempest-share-description")
        # create share_type
        extra_specs = {}
        if CONF.share.capability_snapshot_support:
            extra_specs.update({'snapshot_support': True})
        cls.share_type = cls.create_share_type(extra_specs=extra_specs)
        cls.share_type_id = cls.share_type['id']
        # create share
        cls.share = cls.create_share(
            name=cls.share_name,
            description=cls.share_desc,
            share_type_id=cls.share_type_id)
        if CONF.share.run_snapshot_tests:
            # create snapshot
            cls.snap_name = data_utils.rand_name("tempest-snapshot-name")
            cls.snap_desc = data_utils.rand_name(
                "tempest-snapshot-description")
            cls.snap = cls.create_snapshot_wait_for_active(
                cls.share["id"], cls.snap_name, cls.snap_desc)

    @decorators.idempotent_id('c4481ba3-0cff-448b-a728-69a9a34e3aa6')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    @testtools.skipUnless(
        CONF.share.run_extend_tests,
        "Share extend tests are disabled.")
    @testtools.skipUnless(
        CONF.share.run_quota_tests,
        "Quota tests are disabled.")
    def test_share_extend_over_quota(self):
        tenant_quotas = self.shares_client.show_quotas(
            self.shares_client.tenant_id)['quota_set']
        new_size = int(tenant_quotas["gigabytes"]) + 1

        # extend share with over quota and check result
        self.assertRaises(lib_exc.Forbidden,
                          self.shares_client.extend_share,
                          self.share['id'],
                          new_size)

    @decorators.idempotent_id('3448cd2b-34eb-453f-b72f-39fbea778e42')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    @testtools.skipUnless(
        CONF.share.run_extend_tests,
        "Share extend tests are disabled.")
    def test_share_extend_with_less_size(self):
        new_size = int(self.share['size']) - 1

        # extend share with invalid size and check result
        self.assertRaises(lib_exc.BadRequest,
                          self.shares_client.extend_share,
                          self.share['id'],
                          new_size)

    @decorators.idempotent_id('79f2304a-7959-4169-8a76-b67814e0733a')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    @testtools.skipUnless(
        CONF.share.run_extend_tests,
        "Share extend tests are disabled.")
    def test_share_extend_with_same_size(self):
        new_size = int(self.share['size'])

        # extend share with invalid size and check result
        self.assertRaises(lib_exc.BadRequest,
                          self.shares_client.extend_share,
                          self.share['id'],
                          new_size)

    @decorators.idempotent_id('067c4b10-4324-45ac-8365-5d446b66c18a')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    @testtools.skipUnless(
        CONF.share.run_extend_tests,
        "Share extend tests are disabled.")
    def test_share_extend_with_invalid_share_state(self):
        share = self.create_share(share_type_id=self.share_type_id,
                                  cleanup_in_class=False)
        new_size = int(share['size']) + 1

        # set "error" state
        self.admin_client.reset_state(share['id'])

        # run extend operation on same share and check result
        self.assertRaises(lib_exc.BadRequest,
                          self.shares_client.extend_share,
                          share['id'],
                          new_size)

    @decorators.idempotent_id('f9d2ba94-4032-d17a-b4ab-a2b67f650a39')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    @utils.skip_if_microversion_not_supported("2.64")
    @testtools.skipUnless(
        CONF.share.run_extend_tests,
        "Share extend tests are disabled.")
    def test_share_force_extend_non_admin_user(self):
        # only admin cloud force extend share with micversion >= 2.64
        # non-admin will get unauthorized error.
        new_size = int(self.share['size']) + 1
        self.assertRaises(lib_exc.Forbidden,
                          self.shares_v2_client.extend_share, self.share['id'],
                          new_size, force=True)

    @decorators.idempotent_id('99d42f94-8da1-4c04-ad5b-9738d6acc139')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    @testtools.skipUnless(
        CONF.share.run_shrink_tests,
        "Share shrink tests are disabled.")
    def test_share_shrink_with_greater_size(self):
        new_size = int(self.share['size']) + 1

        # shrink share with invalid size and check result
        self.assertRaises(lib_exc.BadRequest,
                          self.shares_client.shrink_share,
                          self.share['id'],
                          new_size)

    @decorators.idempotent_id('3d4c8f34-49b8-4628-b1cb-652ae67473a5')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    @testtools.skipUnless(
        CONF.share.run_shrink_tests,
        "Share shrink tests are disabled.")
    def test_share_shrink_with_same_size(self):
        new_size = int(self.share['size'])

        # shrink share with invalid size and check result
        self.assertRaises(lib_exc.BadRequest,
                          self.shares_client.shrink_share,
                          self.share['id'],
                          new_size)

    @decorators.idempotent_id('d53ece5c-70e4-4953-a1d7-7d4384510519')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    @testtools.skipUnless(
        CONF.share.run_shrink_tests,
        "Share shrink tests are disabled.")
    def test_share_shrink_with_invalid_share_state(self):
        size = CONF.share.share_size + 1
        share = self.create_share(share_type_id=self.share_type_id,
                                  size=size,
                                  cleanup_in_class=False)
        new_size = int(share['size']) - 1

        # set "error" state
        self.admin_client.reset_state(share['id'])

        # run shrink operation on same share and check result
        self.assertRaises(lib_exc.BadRequest,
                          self.shares_client.shrink_share,
                          share['id'],
                          new_size)

    @decorators.idempotent_id('ff307c91-3bb9-48b5-926c-5a2747320151')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    @utils.skip_if_microversion_not_supported("2.34")
    @ddt.data('path', 'id')
    def test_list_shares_with_export_location_and_invalid_version(
            self, export_location_type):
        # In API versions <v2.35, querying the share API by export
        # location path or ID should have no effect. Those filters were
        # supported from v2.35
        filters = {
            'export_location_' + export_location_type: 'fake',
        }
        shares = self.shares_v2_client.list_shares(
            params=filters, version="2.34")['shares']

        self.assertGreater(len(shares), 0)

    @decorators.idempotent_id('ffc3dc76-2f92-4308-a125-1d3905ed72ba')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    @utils.skip_if_microversion_not_supported("2.35")
    @ddt.data('path', 'id')
    def test_list_shares_with_export_location_not_exist(
            self, export_location_type):
        filters = {
            'export_location_' + export_location_type: 'fake_not_exist',
        }
        shares = self.shares_v2_client.list_shares(
            params=filters)['shares']

        self.assertEqual(0, len(shares))

    @decorators.idempotent_id('3dbcf17b-cc63-43ea-b45f-eae12300729e')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    @utils.skip_if_microversion_not_supported("2.36")
    def test_list_shares_with_like_filter_and_invalid_version(self):
        # In API versions < v2.36, querying the share API by inexact
        # filter (name or description) should have no effect. Those
        # filters were supported from v2.36
        filters = {
            'name~': 'fake',
            'description~': 'fake',
        }
        shares = self.shares_v2_client.list_shares(
            params=filters, version="2.35")['shares']

        self.assertGreater(len(shares), 0)

    @decorators.idempotent_id('f41c6cd2-62cf-4bba-a26e-21a6e86eae15')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    @utils.skip_if_microversion_not_supported("2.36")
    def test_list_shares_with_like_filter_not_exist(self):
        filters = {
            'name~': 'fake_not_exist',
            'description~': 'fake_not_exist',
        }
        shares = self.shares_v2_client.list_shares(params=filters)['shares']

        self.assertEqual(0, len(shares))

    @decorators.idempotent_id('31e33495-5ec3-4658-bdef-d9d1e034705a')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    def test_list_shares_with_name_not_exist(self):
        filters = {
            'name': "tempest-share",
        }
        shares = self.shares_v2_client.list_shares(params=filters)['shares']

        self.assertEqual(0, len(shares))

    @decorators.idempotent_id('5b0ceae1-357f-4b51-81a6-88973ea20c16')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    @utils.skip_if_microversion_not_supported("2.36")
    def test_list_shares_with_description_not_exist(self):
        filters = {
            'description': "tempest-share",
        }
        shares = self.shares_v2_client.list_shares(params=filters)['shares']

        self.assertEqual(0, len(shares))

    @decorators.idempotent_id('061ee37a-96b2-4b4f-9cfe-2c8c80ed4370')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    @utils.skip_if_microversion_not_supported("2.36")
    def test_list_snapshots_with_description_not_exist(self):
        filters = {
            'description': "tempest-snapshot",
        }
        shares = self.shares_v2_client.list_snapshots_with_detail(
            params=filters)['snapshots']

        self.assertEqual(0, len(shares))

    @decorators.idempotent_id('9d3c3158-1a92-4e37-b00f-a4a40b813109')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    def test_list_snapshots_with_name_not_exist(self):
        filters = {
            'name': "tempest-snapshot",
        }
        shares = self.shares_v2_client.list_snapshots_with_detail(
            params=filters)['snapshots']

        self.assertEqual(0, len(shares))

    @decorators.idempotent_id('e8f857f1-ec32-4f81-9e09-26065891dc93')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    def test_get_share_from_other_project(self):
        self.assertRaises(lib_exc.NotFound,
                          self.alt_shares_v2_client.get_share,
                          self.share['id'])

    @utils.skip_if_microversion_not_supported("2.69")
    @decorators.idempotent_id('36cbe23b-08d2-49d9-bb42-f9eb2a804cb1')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    def test_soft_delete_share_has_been_soft_deleted(self):
        share = self.create_share(share_type_id=self.share_type_id,
                                  cleanup_in_class=False)

        # soft delete the share
        self.shares_v2_client.soft_delete_share(share['id'])

        # try soft delete the share again
        self.assertRaises(lib_exc.Forbidden,
                          self.shares_v2_client.soft_delete_share,
                          share['id'])

        # restore the share for resource_cleanup
        self.shares_v2_client.restore_share(share['id'])
        waiters.wait_for_restore(self.shares_v2_client, share['id'])

    @utils.skip_if_microversion_not_supported("2.69")
    @decorators.idempotent_id('cf675ac9-0970-49fc-a051-8a94555c73b5')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    def test_soft_delete_share_with_invalid_share_state(self):
        share = self.create_share(share_type_id=self.share_type_id,
                                  cleanup_in_class=False)

        # set "error_deleting" state
        self.admin_client.reset_state(share['id'], status="error_deleting")

        # try soft delete the share
        self.assertRaises(lib_exc.Forbidden,
                          self.shares_v2_client.soft_delete_share,
                          share['id'])

        # rollback to available status
        self.admin_client.reset_state(share['id'], status="available")

    @utils.skip_if_microversion_not_supported("2.69")
    @decorators.idempotent_id('f6106ee4-1a01-444f-b623-912a5e751d49')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    def test_soft_delete_share_from_other_project(self):
        share = self.create_share(share_type_id=self.share_type_id,
                                  cleanup_in_class=False)

        # NOTE(gouthamr): The client used below is of a member from alt
        # project. As a fix to bug #1901210, the server responds with
        # 404 instead of 403, but we'll test for one of the two codes since
        # the test could be running against a release without the fix.
        self.assertRaises((lib_exc.Forbidden, lib_exc.NotFound),
                          self.alt_shares_v2_client.soft_delete_share,
                          share['id'])

    @utils.skip_if_microversion_not_supported("2.69")
    @decorators.idempotent_id('0ccd44dd-2fda-403e-bc23-7ce428550f36')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_soft_delete_share_with_wrong_id(self):
        self.assertRaises(lib_exc.NotFound,
                          self.shares_v2_client.soft_delete_share,
                          "wrong_share_id")

    @utils.skip_if_microversion_not_supported("2.69")
    @decorators.idempotent_id('87345725-f187-4d7d-86b1-62284e8c75ae')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_restore_share_with_wrong_id(self):
        self.assertRaises(lib_exc.NotFound,
                          self.shares_v2_client.restore_share,
                          "wrong_share_id")



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import datetime

from oslo_utils import timeutils
from oslo_utils import uuidutils
from tempest import config
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc

from manila_tempest_tests.tests.api import base
from manila_tempest_tests import utils

CONF = config.CONF

LOCKS_MIN_API_VERSION = '2.81'

RESOURCE_LOCK_FIELDS = {
    'id',
    'resource_id',
    'resource_action',
    'resource_type',
    'user_id',
    'project_id',
    'lock_context',
    'created_at',
    'updated_at',
    'lock_reason',
    'links',
}


class ResourceLockCRUTest(base.BaseSharesMixedTest):

    @classmethod
    def skip_checks(cls):
        super(ResourceLockCRUTest, cls).skip_checks()
        utils.check_skip_if_microversion_not_supported(LOCKS_MIN_API_VERSION)

    @classmethod
    def resource_setup(cls):
        super(ResourceLockCRUTest, cls).resource_setup()
        # create share type
        share_type = cls.create_share_type()
        cls.share_type_id = share_type['id']

        # create share and place a "delete" lock on it
        cls.share = cls.create_share(share_type_id=cls.share_type_id)
        cls.lock = cls.create_resource_lock(cls.share['id'])

    @decorators.attr(type=[base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND])
    @decorators.idempotent_id('f3d162a6-2ab4-433b-b8e7-6bf4f0bb6b0e')
    def test_list_resource_locks(self):
        locks = self.shares_v2_client.list_resource_locks()['resource_locks']
        self.assertIsInstance(locks, list)
        self.assertIn(self.lock['id'], [x['id'] for x in locks])
        lock = locks[0]
        self.assertEqual(RESOURCE_LOCK_FIELDS, set(lock.keys()))

    @decorators.attr(type=[base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND])
    @decorators.idempotent_id('72cc0d43-f676-4dd8-8a93-faa71608de98')
    def test_list_resource_locks_sorted_and_paginated(self):
        lock_2 = self.create_resource_lock(self.share['id'],
                                           cleanup_in_class=False)
        lock_3 = self.create_resource_lock(self.share['id'],
                                           cleanup_in_class=False)

        expected_order = [self.lock['id'], lock_2['id']]

        filters = {'sort_key': 'created_at', 'sort_dir': 'asc', 'limit': 2}
        body = self.shares_v2_client.list_resource_locks(filters=filters)
        # tempest/lib/common/rest_client.py's _parse_resp checks
        # for number of keys in response's dict, if there is only single
        # key, it returns directly this key, otherwise it returns
        # parsed body. If limit param is used, then API returns
        # multiple keys in response ('resource_locks' and
        # 'resource_lock_links')
        locks = body['resource_locks']
        self.assertIsInstance(locks, list)
        actual_order = [x['id'] for x in locks]
        self.assertEqual(2, len(actual_order))
        self.assertNotIn(lock_3['id'], actual_order)
        self.assertEqual(expected_order, actual_order)

    @decorators.attr(type=[base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND])
    @decorators.idempotent_id('22831edc-9d99-432d-a0b6-85af8853db98')
    def test_list_resource_locks_filtered(self):
        # Filter by resource_id, resource_action, lock_reason_like,
        # created_since, created_before
        share_2 = self.create_share(share_type_id=self.share_type_id)
        share_1_lock_2 = self.create_resource_lock(
            self.share['id'],
            lock_reason="clemson tigers rule",
            cleanup_in_class=False)
        share_2_lock = self.create_resource_lock(share_2['id'],
                                                 cleanup_in_class=False)

        # filter by resource_type
        expected_locks = sorted([
            self.lock['id'],
            share_1_lock_2['id'],
            share_2_lock['id']
        ])
        actual_locks = self.shares_v2_client.list_resource_locks(
            filters={'resource_type': 'share'})['resource_locks']
        self.assertEqual(expected_locks,
                         sorted([lock['id'] for lock in actual_locks]))

        # filter by resource_id
        expected_locks = sorted([self.lock['id'], share_1_lock_2['id']])
        actual_locks = self.shares_v2_client.list_resource_locks(
            filters={'resource_id': self.share['id']})['resource_locks']
        self.assertEqual(expected_locks,
                         sorted([lock['id'] for lock in actual_locks]))

        # filter by inexact lock reason
        actual_locks = self.shares_v2_client.list_resource_locks(
            filters={'lock_reason~': "clemson"})['resource_locks']
        self.assertEqual([share_1_lock_2['id']],
                         [lock['id'] for lock in actual_locks])

        # timestamp filters
        created_at_1 = timeutils.parse_strtime(self.lock['created_at'])
        created_at_2 = timeutils.parse_strtime(share_2_lock['created_at'])
        time_1 = created_at_1 - datetime.timedelta(seconds=1)
        time_2 = created_at_2 - datetime.timedelta(microseconds=1)
        filters_1 = {'created_since': str(time_1)}

        # should return all resource locks created by this test including
        # self.lock
        actual_locks = self.shares_v2_client.list_resource_locks(
            filters=filters_1)['resource_locks']
        actual_lock_ids = [lock['id'] for lock in actual_locks]
        self.assertGreaterEqual(len(actual_lock_ids), 3)
        self.assertIn(self.lock['id'], actual_lock_ids)
        self.assertIn(share_1_lock_2['id'], actual_lock_ids)

        for lock in actual_locks:
            time_diff_with_created_since = timeutils.delta_seconds(
                time_1, timeutils.parse_strtime(lock['created_at']))
            self.assertGreaterEqual(time_diff_with_created_since, 0)

        filters_2 = {
            'created_since': str(time_1),
            'created_before': str(time_2),
        }

        actual_locks = self.shares_v2_client.list_resource_locks(
            filters=filters_2)['resource_locks']
        self.assertIsInstance(actual_locks, list)
        actual_lock_ids = [lock['id'] for lock in actual_locks]
        self.assertGreaterEqual(len(actual_lock_ids), 2)
        self.assertIn(self.lock['id'], actual_lock_ids)
        self.assertIn(share_1_lock_2['id'], actual_lock_ids)
        self.assertNotIn(share_2_lock['id'], actual_lock_ids)

    @decorators.attr(type=[base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND])
    @decorators.idempotent_id('8cbf7331-f3a1-4c7b-ab1e-f8b938bf135e')
    def test_get_resource_lock(self):
        lock = self.shares_v2_client.get_resource_lock(
            self.lock['id'])['resource_lock']

        self.assertEqual(set(RESOURCE_LOCK_FIELDS), set(lock.keys()))
        self.assertTrue(uuidutils.is_uuid_like(lock['id']))
        self.assertEqual('share', lock['resource_type'])
        self.assertEqual(self.share['id'], lock['resource_id'])
        self.assertEqual('delete', lock['resource_action'])
        self.assertEqual('user', lock['lock_context'])
        self.assertEqual(self.shares_v2_client.user_id, lock['user_id'])
        self.assertEqual(self.shares_v2_client.project_id, lock['project_id'])

    @decorators.attr(type=[base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND])
    @decorators.idempotent_id('a7f0fb6a-05ac-4afa-b8d9-04d20549bbd1')
    def test_create_resource_lock(self):
        # testing lock creation by a different user in the same project
        project = self.os_admin.projects_client.show_project(
            self.shares_v2_client.project_id)['project']
        new_user_client = self.create_user_and_get_client(project)

        lock = self.create_resource_lock(
            self.share['id'],
            client=new_user_client.shares_v2_client,
            cleanup_in_class=False)

        self.assertEqual(set(RESOURCE_LOCK_FIELDS), set(lock.keys()))
        self.assertTrue(uuidutils.is_uuid_like(lock['id']))
        self.assertEqual('share', lock['resource_type'])
        self.assertEqual(self.share['id'], lock['resource_id'])
        self.assertEqual('delete', lock['resource_action'])
        self.assertEqual('user', lock['lock_context'])
        self.assertEqual(new_user_client.shares_v2_client.user_id,
                         lock['user_id'])
        self.assertEqual(self.shares_v2_client.project_id, lock['project_id'])

        # testing lock creation by admin
        lock = self.create_resource_lock(
            self.share['id'],
            client=self.admin_shares_v2_client,
            cleanup_in_class=False)
        self.assertEqual('admin', lock['lock_context'])

    @decorators.attr(type=[base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND])
    @decorators.idempotent_id('d7b51cde-ff4f-45ce-a237-401e8be5b4e5')
    def test_update_resource_lock(self):
        lock = self.shares_v2_client.update_resource_lock(
            self.lock['id'], lock_reason="new lock reason")['resource_lock']

        # update is synchronous
        self.assertEqual("new lock reason", lock['lock_reason'])

        # verify get
        lock = self.shares_v2_client.get_resource_lock(lock['id'])
        self.assertEqual("new lock reason",
                         lock['resource_lock']['lock_reason'])


class ResourceLockDeleteTest(base.BaseSharesMixedTest):

    @classmethod
    def skip_checks(cls):
        super(ResourceLockDeleteTest, cls).skip_checks()
        utils.check_skip_if_microversion_not_supported(LOCKS_MIN_API_VERSION)

    @classmethod
    def resource_setup(cls):
        super(ResourceLockDeleteTest, cls).resource_setup()
        cls.share_type_id = cls.create_share_type()['id']

    @decorators.idempotent_id('835fd617-4600-40a0-9ba1-40e5e0097b01')
    @decorators.attr(type=[base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND])
    def test_delete_lock(self):
        share = self.create_share(share_type_id=self.share_type_id)
        lock_1 = self.create_resource_lock(share['id'], cleanup_in_class=False)
        lock_2 = self.create_resource_lock(share['id'], cleanup_in_class=False)

        locks = self.shares_v2_client.list_resource_locks(
            filters={'resource_id': share['id']})['resource_locks']
        self.assertEqual(sorted([lock_1['id'], lock_2['id']]),
                         sorted([lock['id'] for lock in locks]))

        self.shares_v2_client.delete_resource_lock(lock_1['id'])
        locks = self.shares_v2_client.list_resource_locks(
            filters={'resource_id': share['id']})['resource_locks']
        self.assertEqual(1, len(locks))
        self.assertIn(lock_2['id'], [lock['id'] for lock in locks])

    @decorators.idempotent_id('a96e70c7-0afe-4335-9abc-4b45ef778bd7')
    @decorators.attr(type=[base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND])
    def test_delete_locked_resource(self):
        share = self.create_share(share_type_id=self.share_type_id)
        lock_1 = self.create_resource_lock(share['id'], cleanup_in_class=False)
        lock_2 = self.create_resource_lock(share['id'], cleanup_in_class=False)

        # share can't be deleted when a lock exists
        self.assertRaises(lib_exc.Forbidden,
                          self.shares_v2_client.delete_share,
                          share['id'])

        # admin can't do this either
        self.assertRaises(lib_exc.Forbidden,
                          self.admin_shares_v2_client.delete_share,
                          share['id'])
        # "the force" shouldn't work either
        self.assertRaises(lib_exc.Forbidden,
                          self.admin_shares_v2_client.delete_share,
                          share['id'],
                          params={'force': True})

        self.shares_v2_client.delete_resource_lock(lock_1['id'])

        # there's at least one lock, share deletion should still fail
        self.assertRaises(lib_exc.Forbidden,
                          self.shares_v2_client.delete_share,
                          share['id'])

        self.shares_v2_client.delete_resource_lock(lock_2['id'])

        # locks are gone, share deletion should be possible
        self.shares_v2_client.delete_share(share['id'])
        self.shares_v2_client.wait_for_resource_deletion(
            share_id=share["id"])



# Copyright 2015 Deutsche Telekom AG
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import ddt
from oslo_log import log as logging
from tempest import config
from tempest.lib import decorators
from tempest.lib import exceptions
import testtools
from testtools import testcase as tc

from manila_tempest_tests.common import constants
from manila_tempest_tests.tests.api import base
from manila_tempest_tests.tests.scenario import manager_share as manager
from manila_tempest_tests import utils


CONF = config.CONF
LOG = logging.getLogger(__name__)


@ddt.ddt
class ShareBasicOpsBase(manager.ShareScenarioTest):

    """This smoke test case follows this basic set of operations:

     * Create share network
     * Create share
     * Launch an instance
     * Allow access
     * Perform ssh to instance
     * Mount share
     * Terminate the instance
    """

    @decorators.idempotent_id('825be71c-cf14-4884-a0ad-cf47d511df9a')
    @tc.attr(base.TAG_POSITIVE, base.TAG_BACKEND)
    def test_mount_share_one_vm(self):
        instance = self.boot_instance(wait_until="BUILD")
        share = self.create_share()
        locations = self.get_user_export_locations(share)
        instance = self.wait_for_active_instance(instance["id"])
        remote_client = self.init_remote_client(instance)
        self.allow_access(share=share, instance=instance,
                          remote_client=remote_client, locations=locations)

        for location in locations:
            self.mount_share(location, remote_client)
            self.unmount_share(remote_client)

    @decorators.idempotent_id('7cc61131-90e1-42fb-9f07-d3786efb338f')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_BACKEND)
    def test_write_with_ro_access(self):
        '''Test if an instance with ro access can write on the share.'''
        test_data = "Some test data to write"

        instance = self.boot_instance(wait_until="BUILD")
        share = self.create_share()
        location = self.get_user_export_locations(share)[0]
        instance = self.wait_for_active_instance(instance["id"])

        remote_client_inst = self.init_remote_client(instance)

        # First, check if write works RW access.
        acc_rule_id = self.allow_access(
            share=share, instance=instance, remote_client=remote_client_inst,
            locations=location)['id']

        self.mount_share(location, remote_client_inst)
        self.write_data_to_mounted_share(test_data, remote_client_inst)
        self.deny_access(share['id'], acc_rule_id)

        self.allow_access(share=share, instance=instance,
                          remote_client=remote_client_inst, locations=location,
                          access_level='ro')

        self.addCleanup(self.unmount_share, remote_client_inst)

        # Test if write with RO access fails.
        self.assertRaises(exceptions.SSHExecCommandFailed,
                          self.write_data_to_mounted_share,
                          test_data, remote_client_inst)

    @decorators.idempotent_id('5e184576-c7d1-4c16-9b7c-bc9bcd65ba58')
    @tc.attr(base.TAG_POSITIVE, base.TAG_BACKEND)
    def test_read_write_two_vms(self):
        """Boots two vms and writes/reads data on it."""
        test_data = "Some test data to write"

        # Boot two VMs and create share
        instance1 = self.boot_instance(wait_until="BUILD")
        instance2 = self.boot_instance(wait_until="BUILD")
        share = self.create_share()
        location = self.get_user_export_locations(share)[0]
        instance1 = self.wait_for_active_instance(instance1["id"])
        instance2 = self.wait_for_active_instance(instance2["id"])

        # Write data to first VM
        remote_client_inst1 = self.init_remote_client(instance1)
        access = self.allow_access(share=share, instance=instance1,
                                   remote_client=remote_client_inst1,
                                   locations=location)

        self.mount_share(location, remote_client_inst1)
        self.addCleanup(self.unmount_share,
                        remote_client_inst1)
        self.write_data_to_mounted_share(test_data, remote_client_inst1)

        # Read from second VM
        remote_client_inst2 = self.init_remote_client(instance2)
        if not CONF.share.override_ip_for_nfs_access or self.ipv6_enabled:
            self.allow_access(share=share, instance=instance2,
                              remote_client=remote_client_inst2,
                              locations=location,
                              access_rule=access)

        self.mount_share(location, remote_client_inst2)
        self.addCleanup(self.unmount_share,
                        remote_client_inst2)
        data = self.read_data_from_mounted_share(remote_client_inst2)
        self.assertEqual(test_data, data)

    @decorators.idempotent_id('15d42949-545e-4ad8-b06e-bb2556c54375')
    @tc.attr(base.TAG_POSITIVE, base.TAG_BACKEND)
    @utils.skip_if_microversion_not_supported("2.29")
    @testtools.skipUnless(CONF.share.run_host_assisted_migration_tests or
                          CONF.share.run_driver_assisted_migration_tests,
                          "Share migration tests are disabled.")
    @ddt.data(True, False)
    def test_migration_files(self, force_host_assisted):

        if (force_host_assisted and
                not CONF.share.run_host_assisted_migration_tests):
            raise self.skipException("Host-assisted migration tests are "
                                     "disabled.")
        elif (not force_host_assisted and
              not CONF.share.run_driver_assisted_migration_tests):
            raise self.skipException("Driver-assisted migration tests are "
                                     "disabled.")

        if self.protocol != "nfs":
            raise self.skipException("Only NFS protocol supported "
                                     "at this moment.")

        if self.ipv6_enabled:
            raise self.skipException("Share Migration using IPv6 is not "
                                     "supported at this moment.")

        pools = self.shares_admin_v2_client.list_pools(detail=True)['pools']

        if len(pools) < 2:
            raise self.skipException("At least two different pool entries are "
                                     "needed to run share migration tests.")

        instance = self.boot_instance(wait_until="BUILD")
        share = self.create_share()
        export_location = self.get_user_export_locations(share)[0]
        instance = self.wait_for_active_instance(instance["id"])
        share = self.shares_admin_v2_client.get_share(share['id'])['share']

        default_type = self.shares_v2_client.list_share_types(
            default=True)['share_type']

        dest_pool = utils.choose_matching_backend(share, pools, default_type)

        self.assertIsNotNone(dest_pool)
        self.assertIsNotNone(dest_pool.get('name'))

        dest_pool = dest_pool['name']

        remote_client = self.init_remote_client(instance)

        self.allow_access(share=share,
                          instance=instance,
                          remote_client=remote_client,
                          locations=export_location)

        self.mount_share(export_location, remote_client)

        remote_client.exec_command("sudo mkdir -p /mnt/f1")
        remote_client.exec_command("sudo mkdir -p /mnt/f2")
        remote_client.exec_command("sudo mkdir -p /mnt/f3")
        remote_client.exec_command("sudo mkdir -p /mnt/f4")
        remote_client.exec_command("sudo mkdir -p /mnt/f1/ff1")
        remote_client.exec_command("sleep 1")
        remote_client.exec_command(
            "sudo dd if=/dev/zero of=/mnt/f1/1m1.bin bs=1M count=1")
        remote_client.exec_command(
            "sudo dd if=/dev/zero of=/mnt/f2/1m2.bin bs=1M count=1")
        remote_client.exec_command(
            "sudo dd if=/dev/zero of=/mnt/f3/1m3.bin bs=1M count=1")
        remote_client.exec_command(
            "sudo dd if=/dev/zero of=/mnt/f4/1m4.bin bs=1M count=1")
        remote_client.exec_command(
            "sudo dd if=/dev/zero of=/mnt/f1/ff1/1m5.bin bs=1M count=1")
        remote_client.exec_command("sudo chmod -R 555 /mnt/f3")
        remote_client.exec_command("sudo chmod -R 777 /mnt/f4")

        task_state = (constants.TASK_STATE_DATA_COPYING_COMPLETED
                      if force_host_assisted
                      else constants.TASK_STATE_MIGRATION_DRIVER_PHASE1_DONE)

        share = self.migrate_share(
            share['id'], dest_pool, task_state, force_host_assisted)

        if force_host_assisted:
            self.assertRaises(
                exceptions.SSHExecCommandFailed,
                remote_client.exec_command,
                "dd if=/dev/zero of=/mnt/f1/1m6.bin bs=1M count=1")

        self.unmount_share(remote_client)

        share = self.migration_complete(share['id'], dest_pool)

        new_exports = self.get_user_export_locations(share)

        self.assertEqual(dest_pool, share['host'])
        self.assertEqual(constants.TASK_STATE_MIGRATION_SUCCESS,
                         share['task_state'])

        self.mount_share(new_exports[0], remote_client)

        output = remote_client.exec_command("ls -lRA --ignore=lost+found /mnt")

        self.unmount_share(remote_client)

        self.assertIn('1m1.bin', output)
        self.assertIn('1m2.bin', output)
        self.assertIn('1m3.bin', output)
        self.assertIn('1m4.bin', output)
        self.assertIn('1m5.bin', output)

    @decorators.idempotent_id('87b803bf-679a-492b-a538-af4c9ff013c8')
    @tc.attr(base.TAG_POSITIVE, base.TAG_BACKEND)
    @testtools.skipUnless(
        CONF.share.capability_create_share_from_snapshot_support,
        "Create share from snapshot tests are disabled.")
    def test_write_data_to_share_created_from_snapshot(self):
        # 1 - Create UVM, ok, created
        instance = self.boot_instance(wait_until="BUILD")

        # 2 - Create share S1, ok, created
        extra_specs = {'snapshot_support': True}
        parent_share = self.create_share(extra_specs=extra_specs)
        parent_share_export_location = self.get_user_export_locations(
            parent_share)[0]

        # Create a client User Virtual Machine
        instance = self.wait_for_active_instance(instance["id"])
        self.addCleanup(self.servers_client.delete_server, instance['id'])

        # 3 - SSH to UVM, ok, connected
        remote_client = self.init_remote_client(instance)

        # 4 - Provide RW access to S1, ok, provided
        self.allow_access(share=parent_share,
                          instance=instance,
                          remote_client=remote_client,
                          locations=parent_share_export_location)

        # 5 - Try mount S1 to UVM, ok, mounted

        parent_share_dir = "/mnt/parent"
        remote_client.exec_command("sudo mkdir -p %s" % parent_share_dir)

        self.mount_share(parent_share_export_location,
                         remote_client,
                         parent_share_dir)
        self.addCleanup(self.unmount_share, remote_client, parent_share_dir)

        # 6 - Create "file1", ok, created
        remote_client.exec_command("sudo touch %s/file1" % parent_share_dir)

        # 7 - Create snapshot SS1 from S1, ok, created
        snapshot = self._create_snapshot(parent_share['id'])

        # 8 - Create "file2" in share S1 - ok, created. We expect that
        # snapshot will not contain any data created after snapshot creation.
        remote_client.exec_command("sudo touch %s/file2" % parent_share_dir)

        # 9 - Create share S2 from SS1, ok, created
        child_share = self.create_share(snapshot_id=snapshot["id"])

        # 10 - Try mount S2 - fail, access denied. We test that child share
        #      did not get access rules from parent share.
        child_share_export_location = self.get_user_export_locations(
            child_share)[0]
        child_share_dir = "/mnt/child"
        remote_client.exec_command("sudo mkdir -p %s" % child_share_dir)

        self.assertRaises(
            exceptions.SSHExecCommandFailed,
            self.mount_share,
            child_share_export_location, remote_client, child_share_dir,
        )

        # 11 - Provide RW access to S2, ok, provided
        self.allow_access(share=child_share,
                          instance=instance,
                          remote_client=remote_client,
                          locations=child_share_export_location)

        # 12 - Try mount S2, ok, mounted
        self.mount_share(child_share_export_location,
                         remote_client,
                         child_share_dir)
        self.addCleanup(self.unmount_share, remote_client, child_share_dir)

        # 13 - List files on S2, only "file1" exists
        output = remote_client.exec_command(
            "sudo ls -lRA %s" % child_share_dir)
        self.assertIn('file1', output)
        self.assertNotIn('file2', output)

        # 14 - Create file3 on S2, ok, file created
        remote_client.exec_command("sudo touch %s/file3" % child_share_dir)

        # 15 - List files on S1, two files exist - "file1" and "file2"
        output = remote_client.exec_command(
            "sudo ls -lRA %s" % parent_share_dir)
        self.assertIn('file1', output)
        self.assertIn('file2', output)
        self.assertNotIn('file3', output)

        # 16 - List files on S2, two files exist - "file1" and "file3"
        output = remote_client.exec_command(
            "sudo ls -lRA %s" % child_share_dir)
        self.assertIn('file1', output)
        self.assertNotIn('file2', output)
        self.assertIn('file3', output)

    @decorators.idempotent_id('c98e6876-3a4f-40e8-8b4f-023c94c242c3')
    @tc.attr(base.TAG_POSITIVE, base.TAG_BACKEND)
    @utils.skip_if_microversion_not_supported("2.32")
    @testtools.skipUnless(CONF.share.run_mount_snapshot_tests,
                          'Mountable snapshots tests are disabled.')
    @testtools.skipUnless(CONF.share.run_snapshot_tests,
                          "Snapshot tests are disabled.")
    def test_read_mountable_snapshot(self):
        # 1 - Create UVM, ok, created
        instance = self.boot_instance(wait_until="BUILD")

        # 2 - Create share S1, ok, created
        extra_specs = {'snapshot_support': True}
        parent_share = self.create_share(extra_specs=extra_specs)
        user_export_location = self.get_user_export_locations(parent_share)[0]

        # Create client User Virtual Machine
        instance = self.wait_for_active_instance(instance["id"])
        self.addCleanup(self.servers_client.delete_server, instance['id'])

        # 3 - SSH to UVM, ok, connected
        remote_client = self.init_remote_client(instance)

        # 4 - Provide RW access to S1, ok, provided
        self.allow_access(share=parent_share,
                          instance=instance,
                          remote_client=remote_client,
                          locations=user_export_location)

        # 5 - Try mount S1 to UVM, ok, mounted
        parent_share_dir = "/mnt/parent"
        snapshot_dir = "/mnt/snapshot_dir"
        remote_client.exec_command("sudo mkdir -p %s" % parent_share_dir)
        remote_client.exec_command("sudo mkdir -p %s" % snapshot_dir)

        self.mount_share(user_export_location, remote_client, parent_share_dir)
        self.addCleanup(self.unmount_share, remote_client, parent_share_dir)

        # 6 - Create "file1", ok, created
        remote_client.exec_command("sudo touch %s/file1" % parent_share_dir)

        # 7 - Create snapshot SS1 from S1, ok, created
        snapshot = self._create_snapshot(parent_share['id'])
        snapshot_export_location = self.get_user_export_locations(
            snapshot=snapshot)[0]

        # 8 - Create "file2" in share S1 - ok, created. We expect that
        # snapshot will not contain any data created after snapshot creation.
        remote_client.exec_command("sudo touch %s/file2" % parent_share_dir)

        # 9 - Allow access to SS1
        self.allow_access(share=parent_share,
                          instance=instance,
                          snapshot=snapshot,
                          remote_client=remote_client,
                          locations=snapshot_export_location)

        # 10 - Mount SS1
        self.mount_share(snapshot_export_location, remote_client, snapshot_dir)
        self.addCleanup(self.unmount_share, remote_client, snapshot_dir)

        # 11 - List files on SS1, only "file1" exists
        # NOTE(lseki): using ls without recursion to avoid permission denied
        #              error while listing lost+found directory on LVM volumes
        output = remote_client.exec_command("sudo ls -lA %s" % snapshot_dir)
        self.assertIn('file1', output)
        self.assertNotIn('file2', output)

        # 12 - Try to create a file on SS1, should fail
        self.assertRaises(
            exceptions.SSHExecCommandFailed,
            remote_client.exec_command,
            "sudo touch %s/file3" % snapshot_dir)


class TestShareBasicOpsNFS(manager.BaseShareScenarioNFSTest,
                           ShareBasicOpsBase):
    pass


class TestShareBasicOpsCIFS(manager.BaseShareScenarioCIFSTest,
                            ShareBasicOpsBase):

    @decorators.idempotent_id('4344a47a-d316-496b-97a4-12a59297950a')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_BACKEND)
    def test_write_with_ro_access(self):
        msg = ("Skipped for CIFS protocol because RO access is not "
               "supported for shares by IP.")
        raise self.skipException(msg)

    @decorators.idempotent_id('a691332b-dd7a-4041-9bbd-3893e168aefa')
    @tc.attr(base.TAG_POSITIVE, base.TAG_BACKEND)
    def test_read_mountable_snapshot(self):
        msg = "Skipped for CIFS protocol because of bug/1649573"
        raise self.skipException(msg)

    @decorators.idempotent_id('8c936c3e-4793-49d2-8409-4038f03e7012')
    @tc.attr(base.TAG_POSITIVE, base.TAG_BACKEND)
    def test_write_data_to_share_created_from_snapshot(self):
        msg = "Skipped for CIFS protocol because of bug/1649573"
        raise self.skipException(msg)


class TestBaseShareBasicOpsScenarioCEPHFS(manager.BaseShareScenarioCEPHFSTest,
                                          ShareBasicOpsBase):
    @decorators.idempotent_id('9fb12879-45b3-4042-acac-82be338dbde1')
    @tc.attr(base.TAG_POSITIVE, base.TAG_BACKEND)
    def test_mount_share_one_vm_with_ceph_fuse_client(self):
        self.mount_client = 'fuse'
        super(TestBaseShareBasicOpsScenarioCEPHFS,
              self).test_mount_share_one_vm()

    @decorators.idempotent_id('a2a70b94-f5fc-438a-9dfa-53aa60ee3949')
    @tc.attr(base.TAG_POSITIVE, base.TAG_BACKEND)
    def test_write_with_ro_access_with_ceph_fuse_client(self):
        self.mount_client = 'fuse'
        super(TestBaseShareBasicOpsScenarioCEPHFS,
              self).test_write_with_ro_access()

    @decorators.idempotent_id('c247f51f-0ffc-4a4f-894c-781647619faf')
    @tc.attr(base.TAG_POSITIVE, base.TAG_BACKEND)
    def test_read_write_two_vms_with_ceph_fuse_client(self):
        self.mount_client = 'fuse'
        super(TestBaseShareBasicOpsScenarioCEPHFS,
              self).test_read_write_two_vms()

    @decorators.idempotent_id('5bd64c46-05f4-4891-a08f-e146d1a76437')
    @tc.attr(base.TAG_POSITIVE, base.TAG_BACKEND)
    def test_write_data_to_share_created_from_snapshot_with_ceph_fuse_client(
            self):
        self.mount_client = 'fuse'
        super(TestBaseShareBasicOpsScenarioCEPHFS,
              self).test_write_data_to_share_created_from_snapshot()


class TestShareBasicOpsNFSIPv6(TestShareBasicOpsNFS):
    ip_version = 6


# NOTE(u_glide): this function is required to exclude ShareBasicOpsBase from
# executed test cases.
# See: https://docs.python.org/3/library/unittest.html#load-tests-protocol
# for details.
def load_tests(loader, tests, _):
    result = []
    for test_case in tests:
        if type(test_case._tests[0]) is ShareBasicOpsBase:
            continue
        result.append(test_case)
    return loader.suiteClass(result)



# Copyright 2014 Mirantis Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import ddt
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc
import testtools
from testtools import testcase as tc

from manila_tempest_tests.tests.api import base
from manila_tempest_tests import utils

CONF = config.CONF
PRE_SHARE_GROUPS_MICROVERSION = "2.39"
SHARE_GROUPS_MICROVERSION = "2.40"
PRE_SHARE_REPLICA_QUOTAS_MICROVERSION = "2.52"
SHARE_REPLICA_QUOTAS_MICROVERSION = "2.53"


@ddt.ddt
class SharesAdminQuotasTest(base.BaseSharesAdminTest):

    @classmethod
    def skip_checks(cls):
        super(SharesAdminQuotasTest, cls).skip_checks()
        if not CONF.share.run_quota_tests:
            msg = "Quota tests are disabled."
            raise cls.skipException(msg)

    @classmethod
    def resource_setup(cls):
        super(SharesAdminQuotasTest, cls).resource_setup()
        cls.client = cls.shares_v2_client
        cls.user_id = cls.client.user_id
        cls.tenant_id = cls.client.tenant_id
        # create share type
        cls.share_type = cls.create_share_type()
        cls.share_type_id = cls.share_type['id']

    @decorators.idempotent_id('f62c48e3-9736-4f0c-9f9b-f139f393ac0a')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    def test_default_quotas(self):
        quotas = self.client.default_quotas(self.tenant_id)['quota_set']
        self.assertGreater(int(quotas["gigabytes"]), -2)
        self.assertGreater(int(quotas["snapshot_gigabytes"]), -2)
        self.assertGreater(int(quotas["shares"]), -2)
        self.assertGreater(int(quotas["snapshots"]), -2)
        self.assertGreater(int(quotas["share_networks"]), -2)
        if utils.is_microversion_supported(SHARE_GROUPS_MICROVERSION):
            self.assertGreater(int(quotas["share_groups"]), -2)
            self.assertGreater(int(quotas["share_group_snapshots"]), -2)
        if utils.share_replica_quotas_are_supported():
            self.assertGreater(int(quotas["share_replicas"]), -2)
            self.assertGreater(int(quotas["replica_gigabytes"]), -2)

    @decorators.idempotent_id('1ff57cfa-cd8d-495f-86eb-9fead307428e')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    def test_show_quotas(self):
        quotas = self.client.show_quotas(self.tenant_id)['quota_set']
        self.assertGreater(int(quotas["gigabytes"]), -2)
        self.assertGreater(int(quotas["snapshot_gigabytes"]), -2)
        self.assertGreater(int(quotas["shares"]), -2)
        self.assertGreater(int(quotas["snapshots"]), -2)
        self.assertGreater(int(quotas["share_networks"]), -2)
        if utils.is_microversion_supported(SHARE_GROUPS_MICROVERSION):
            self.assertGreater(int(quotas["share_groups"]), -2)
            self.assertGreater(int(quotas["share_group_snapshots"]), -2)
        if utils.share_replica_quotas_are_supported():
            self.assertGreater(int(quotas["share_replicas"]), -2)
            self.assertGreater(int(quotas["replica_gigabytes"]), -2)

    @decorators.idempotent_id('9b96dd45-7c0d-41ee-88e4-600185f61358')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    def test_show_quotas_for_user(self):
        quotas = self.client.show_quotas(
            self.tenant_id, self.user_id)['quota_set']
        self.assertGreater(int(quotas["gigabytes"]), -2)
        self.assertGreater(int(quotas["snapshot_gigabytes"]), -2)
        self.assertGreater(int(quotas["shares"]), -2)
        self.assertGreater(int(quotas["snapshots"]), -2)
        self.assertGreater(int(quotas["share_networks"]), -2)
        if utils.is_microversion_supported(SHARE_GROUPS_MICROVERSION):
            self.assertGreater(int(quotas["share_groups"]), -2)
            self.assertGreater(int(quotas["share_group_snapshots"]), -2)
        if utils.share_replica_quotas_are_supported():
            self.assertGreater(int(quotas["share_replicas"]), -2)
            self.assertGreater(int(quotas["replica_gigabytes"]), -2)

    @decorators.idempotent_id('2e98a13e-b2ed-4977-bafe-47ea48b504f2')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    @utils.skip_if_microversion_not_supported(PRE_SHARE_GROUPS_MICROVERSION)
    def test_show_sg_quotas_using_too_old_microversion(self):
        quotas = self.client.show_quotas(
            self.tenant_id, version=PRE_SHARE_GROUPS_MICROVERSION)['quota_set']

        for key in ('share_groups', 'share_group_snapshots'):
            self.assertNotIn(key, quotas)

    @decorators.idempotent_id('b8bcbc04-68fb-4c8f-9f4c-a3b6c6b8911c')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    @utils.skip_if_microversion_not_supported(PRE_SHARE_GROUPS_MICROVERSION)
    def test_show_sg_quotas_for_user_using_too_old_microversion(self):
        quotas = self.client.show_quotas(
            self.tenant_id, self.user_id,
            version=PRE_SHARE_GROUPS_MICROVERSION)['quota_set']

        for key in ('share_groups', 'share_group_snapshots'):
            self.assertNotIn(key, quotas)

    @decorators.idempotent_id('19fe431b-e83e-4c4e-acb8-018d7a470c8b')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    @utils.skip_if_microversion_not_supported(
        PRE_SHARE_REPLICA_QUOTAS_MICROVERSION)
    def test_show_replica_quotas_for_user_using_too_old_microversion(self):
        quotas = self.client.show_quotas(
            self.tenant_id, self.user_id,
            version=PRE_SHARE_REPLICA_QUOTAS_MICROVERSION)['quota_set']

        for key in ('share_replicas', 'replica_gigabytes'):
            self.assertNotIn(key, quotas)

    @ddt.data(
        ('id', True),
        ('name', False),
    )
    @ddt.unpack
    @decorators.idempotent_id('836e1725-2853-4d54-b281-8173773d8527')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    @utils.skip_if_microversion_not_supported("2.39")
    def test_show_share_type_quotas(self, share_type_key, is_st_public):
        # Check if the used microversion supports 'share_replica' and
        # 'replica_gigabytes' quotas
        replica_quotas_supported = utils.share_replica_quotas_are_supported()

        # Create share type
        share_type = self.create_share_type(
            data_utils.rand_name("tempest-manila"),
            is_public=is_st_public,
            cleanup_in_class=False,
            extra_specs=self.add_extra_specs_to_dict(),
        )
        if 'share_type' in share_type:
            share_type = share_type['share_type']

        keys = ['shares', 'gigabytes', 'snapshots', 'snapshot_gigabytes']

        if replica_quotas_supported:
            keys.append('share_replicas')
            keys.append('replica_gigabytes')

        # Get current project quotas
        p_quotas = self.client.show_quotas(self.tenant_id)['quota_set']

        # Get current share type quotas
        st_quotas = self.client.show_quotas(
            self.tenant_id, share_type=share_type[share_type_key])['quota_set']

        # Share type quotas have values equal to project's
        for key in keys:
            self.assertEqual(st_quotas[key], p_quotas[key])

        # Verify that we do not have share groups related quotas
        # for share types.
        for key in ('share_groups', 'share_group_snapshots'):
            self.assertNotIn(key, st_quotas)


@ddt.ddt
class SharesAdminQuotasUpdateTest(base.BaseSharesAdminTest):

    # We want to force a fresh project for this test class, since we'll be
    # manipulating project quotas - and any pre-existing projects may have
    # resources, quotas and the like that might interfere with our test cases.
    force_tenant_isolation = True

    @classmethod
    def skip_checks(cls):
        super(SharesAdminQuotasUpdateTest, cls).skip_checks()
        if not CONF.auth.use_dynamic_credentials:
            raise cls.skipException('Dynamic credentials are required')
        if not CONF.share.run_quota_tests:
            msg = "Quota tests are disabled."
            raise cls.skipException(msg)

    @classmethod
    def resource_setup(cls):
        super(SharesAdminQuotasUpdateTest, cls).resource_setup()
        # create share type
        extra_specs = {}
        if CONF.share.capability_snapshot_support:
            extra_specs.update({'snapshot_support': True})
        if CONF.share.capability_create_share_from_snapshot_support:
            extra_specs.update({'create_share_from_snapshot_support': True})
        cls.share_type = cls.create_share_type(extra_specs=extra_specs)
        cls.share_type_id = cls.share_type['id']
        # create share group type
        cls.share_group_type = cls._create_share_group_type()
        cls.share_group_type_id = cls.share_group_type['id']
        cls.client = cls.shares_v2_client
        cls.tenant_id = cls.client.tenant_id
        cls.user_id = cls.client.user_id

    @decorators.idempotent_id('da16e906-e8e6-4aa0-9fc1-76ed48cfd428')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    def test_update_tenant_quota_shares(self):
        # get current quotas
        quotas = self.client.show_quotas(self.tenant_id)['quota_set']
        new_quota = int(quotas["shares"]) + 2

        # set new quota for shares
        updated = self.update_quotas(self.tenant_id, shares=new_quota)

        self.assertEqual(new_quota, int(updated["shares"]))

    @ddt.data(
        "share_groups",
        "share_group_snapshots",
    )
    @decorators.idempotent_id('cb09de7e-94e9-401a-b82b-8b2de210f8b9')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    @testtools.skipUnless(
        CONF.share.run_share_group_tests, 'Share Group tests disabled.')
    @utils.skip_if_microversion_not_supported(SHARE_GROUPS_MICROVERSION)
    def test_update_tenant_quota_share_groups(self, quota_key):
        # Get current quotas
        quotas = self.client.show_quotas(self.tenant_id)['quota_set']
        new_quota = int(quotas[quota_key]) + 2

        # Set new quota
        updated = self.update_quotas(self.tenant_id, **{quota_key: new_quota})

        self.assertEqual(new_quota, int(updated[quota_key]))

    @decorators.idempotent_id('2c7f9e19-268d-4420-a046-a7faf21174a1')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    def test_update_user_quota_shares(self):
        # get current quotas
        quotas = self.client.show_quotas(
            self.tenant_id, self.user_id)['quota_set']
        new_quota = int(quotas["shares"]) - 1

        # set new quota for shares
        updated = self.update_quotas(self.tenant_id,
                                     user_id=self.user_id,
                                     shares=new_quota)

        self.assertEqual(new_quota, int(updated["shares"]))

    @ddt.data(
        "share_groups",
        "share_group_snapshots",
    )
    @decorators.idempotent_id('c32a716b-f971-4855-97ea-f30d4423d03d')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    @testtools.skipUnless(
        CONF.share.run_share_group_tests, 'Share Group tests disabled.')
    @utils.skip_if_microversion_not_supported(SHARE_GROUPS_MICROVERSION)
    def test_update_user_quota_share_groups(self, quota_key):
        # Get current quotas
        quotas = self.client.show_quotas(
            self.tenant_id, self.user_id)['quota_set']
        new_quota = int(quotas[quota_key]) - 1

        # Set new quota
        updated = self.update_quotas(self.tenant_id,
                                     user_id=self.user_id,
                                     **{quota_key: new_quota})

        self.assertEqual(new_quota, int(updated[quota_key]))

    @ddt.data(("share_replicas", False),
              ("share_replicas", True),
              ("replica_gigabytes", False),
              ("replica_gigabytes", True),
              )
    @ddt.unpack
    @decorators.idempotent_id('af16dc89-c93d-43de-8902-2c88c75f107f')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    @utils.skip_if_microversion_not_supported(
        SHARE_REPLICA_QUOTAS_MICROVERSION)
    def test_update_user_quota_replica_related(self, quota_key, use_user_id):
        kwargs = {}

        # Update the kwargs with user_id in case the user_id need to be
        # specified in the request
        kwargs.update({'user_id': self.user_id}) if use_user_id else None
        quotas = self.client.show_quotas(self.tenant_id, **kwargs)['quota_set']
        new_quota = int(quotas[quota_key]) - 1

        # Add the updated quota into the kwargs
        kwargs.update({quota_key: new_quota})

        # Set the new quota based on tenant or tenant and user_id
        updated = self.update_quotas(self.tenant_id, **kwargs)

        self.assertEqual(new_quota, int(updated[quota_key]))

    @ddt.data(
        ('id', True),
        ('name', False),
    )
    @ddt.unpack
    @decorators.idempotent_id('155ea3de-b3b5-4aa0-be8b-eebcc19ce874')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    @utils.skip_if_microversion_not_supported("2.39")
    def test_update_share_type_quota(self, share_type_key, is_st_public):
        # Check if the used microversion supports 'share_replica' and
        # 'replica_gigabytes' quotas
        replica_quotas_supported = utils.share_replica_quotas_are_supported()
        share_type = self.create_share_type(is_public=is_st_public)

        # Get current quotas
        quotas = self.client.show_quotas(
            self.tenant_id, share_type=share_type[share_type_key])['quota_set']
        quota_keys = ['shares', 'gigabytes', 'snapshots', 'snapshot_gigabytes']

        if replica_quotas_supported:
            quota_keys.append('share_replicas')
            quota_keys.append('replica_gigabytes')

        # Update quotas
        for q in quota_keys:
            new_quota = int(quotas[q]) - 1

            # Set new quota, cleanup isn't necessary, share type will be
            # deleted when the test concludes, and that'll take care of
            # share type quotas
            updated = self.update_quotas(self.tenant_id,
                                         share_type=share_type[share_type_key],
                                         cleanup=False,
                                         **{q: new_quota})

            self.assertEqual(new_quota, int(updated[q]))

        current_quotas = self.client.show_quotas(
            self.tenant_id, share_type=share_type[share_type_key])['quota_set']

        for q in quota_keys:
            self.assertEqual(int(quotas[q]) - 1, current_quotas[q])

    @decorators.idempotent_id('78957d97-afad-4371-a21e-79641fff83f6')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    def test_update_tenant_quota_snapshots(self):
        # get current quotas
        quotas = self.client.show_quotas(self.tenant_id)['quota_set']
        new_quota = int(quotas["snapshots"]) + 2

        # set new quota for snapshots
        updated = self.update_quotas(self.tenant_id, snapshots=new_quota)

        self.assertEqual(new_quota, int(updated["snapshots"]))

    @decorators.idempotent_id('53f4fd79-39aa-42be-82ce-e423ebffe837')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    def test_update_user_quota_snapshots(self):
        # get current quotas
        quotas = self.client.show_quotas(
            self.tenant_id, self.user_id)['quota_set']
        new_quota = int(quotas["snapshots"]) - 1

        # set new quota for snapshots
        updated = self.update_quotas(self.tenant_id,
                                     user_id=self.user_id,
                                     snapshots=new_quota)

        self.assertEqual(new_quota, int(updated["snapshots"]))

    @decorators.idempotent_id('37ee5bd2-db07-4817-b71a-7c3e78634399')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    def test_update_tenant_quota_gigabytes(self):
        # get current quotas
        custom = self.client.show_quotas(self.tenant_id)['quota_set']

        # make quotas for update
        gigabytes = int(custom["gigabytes"]) + 2

        # set new quota for shares
        updated = self.update_quotas(self.tenant_id, gigabytes=gigabytes)

        self.assertEqual(gigabytes, int(updated["gigabytes"]))

    @decorators.idempotent_id('284a2e95-48a1-4f1b-b952-f734b1b6238a')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    def test_update_tenant_quota_snapshot_gigabytes(self):
        # get current quotas
        custom = self.client.show_quotas(self.tenant_id)['quota_set']

        # make quotas for update
        snapshot_gigabytes = int(custom["snapshot_gigabytes"]) + 2

        # set new quota for shares
        updated = self.update_quotas(self.tenant_id,
                                     snapshot_gigabytes=snapshot_gigabytes)

        self.assertEqual(snapshot_gigabytes,
                         int(updated["snapshot_gigabytes"]))

    @decorators.idempotent_id('75977d53-f06b-41a2-8365-0ce549e4a51a')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    def test_update_user_quota_gigabytes(self):
        # get current quotas
        custom = self.client.show_quotas(
            self.tenant_id, self.user_id)['quota_set']

        # make quotas for update
        gigabytes = int(custom["gigabytes"]) - 1

        # set new quota for shares
        updated = self.update_quotas(self.tenant_id,
                                     user_id=self.user_id,
                                     gigabytes=gigabytes)

        self.assertEqual(gigabytes, int(updated["gigabytes"]))

    @decorators.idempotent_id('00a189fc-93ed-44c2-b9dc-1d9b6c26d005')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    def test_update_user_quota_snapshot_gigabytes(self):
        # get current quotas
        custom = self.client.show_quotas(
            self.tenant_id, self.user_id)['quota_set']

        # make quotas for update
        snapshot_gigabytes = int(custom["snapshot_gigabytes"]) - 1

        # set new quota for shares
        updated = self.update_quotas(self.tenant_id,
                                     user_id=self.user_id,
                                     snapshot_gigabytes=snapshot_gigabytes)

        self.assertEqual(snapshot_gigabytes,
                         int(updated["snapshot_gigabytes"]))

    @decorators.idempotent_id('da7f3179-f2f3-402e-82c2-e6855774a99a')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    def test_update_tenant_quota_share_networks(self):
        # get current quotas
        quotas = self.client.show_quotas(self.tenant_id)['quota_set']
        new_quota = int(quotas["share_networks"]) + 2

        # set new quota for share-networks
        updated = self.update_quotas(self.tenant_id, share_networks=new_quota)

        self.assertEqual(new_quota, int(updated["share_networks"]))

    @decorators.idempotent_id('f75f01a0-5921-44ab-b373-bb9e070f87eb')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    def test_update_user_quota_share_networks(self):
        # get current quotas
        quotas = self.client.show_quotas(
            self.tenant_id, self.user_id)['quota_set']
        new_quota = int(quotas["share_networks"]) - 1

        # set new quota for share-networks
        updated = self.update_quotas(self.tenant_id,
                                     user_id=self.user_id,
                                     share_networks=new_quota)

        self.assertEqual(new_quota, int(updated["share_networks"]))

    @decorators.idempotent_id('84e24c32-ee78-461e-ac1f-f9e4d99f88e2')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    def test_reset_tenant_quotas(self):
        # Get default_quotas
        default = self.client.default_quotas(self.tenant_id)['quota_set']

        # Get current quotas
        custom = self.client.show_quotas(self.tenant_id)['quota_set']

        # Make quotas for update
        data = {
            "shares": int(custom["shares"]) + 2,
            "snapshots": int(custom["snapshots"]) + 2,
            "gigabytes": int(custom["gigabytes"]) + 2,
            "snapshot_gigabytes": int(custom["snapshot_gigabytes"]) + 2,
            "share_networks": int(custom["share_networks"]) + 2,
        }
        if (utils.is_microversion_supported(SHARE_GROUPS_MICROVERSION) and
                CONF.share.run_share_group_tests):
            data["share_groups"] = int(custom["share_groups"]) + 2
            data["share_group_snapshots"] = (
                int(custom["share_group_snapshots"]) + 2)
        if utils.share_replica_quotas_are_supported():
            data["share_replicas"] = int(custom["share_replicas"]) + 2
            data["replica_gigabytes"] = int(custom["replica_gigabytes"]) + 2

        # set new quota, turn off cleanup - we'll do it right below
        updated = self.update_quotas(self.tenant_id, cleanup=False, **data)
        self.assertEqual(data["shares"], int(updated["shares"]))
        self.assertEqual(data["snapshots"], int(updated["snapshots"]))
        self.assertEqual(data["gigabytes"], int(updated["gigabytes"]))
        self.assertEqual(
            data["snapshot_gigabytes"], int(updated["snapshot_gigabytes"]))
        self.assertEqual(
            data["share_networks"], int(updated["share_networks"]))
        if (utils.is_microversion_supported(SHARE_GROUPS_MICROVERSION) and
                CONF.share.run_share_group_tests):
            self.assertEqual(
                data["share_groups"], int(updated["share_groups"]))
            self.assertEqual(
                data["share_group_snapshots"],
                int(updated["share_group_snapshots"]))
        if utils.share_replica_quotas_are_supported():
            self.assertEqual(
                data["share_replicas"], int(updated["share_replicas"]))
            self.assertEqual(
                data["replica_gigabytes"], int(updated["replica_gigabytes"]))

        # Reset customized quotas
        self.client.reset_quotas(self.tenant_id)

        # Verify quotas
        reseted = self.client.show_quotas(self.tenant_id)['quota_set']
        self.assertEqual(int(default["shares"]), int(reseted["shares"]))
        self.assertEqual(int(default["snapshots"]), int(reseted["snapshots"]))
        self.assertEqual(int(default["gigabytes"]), int(reseted["gigabytes"]))
        self.assertEqual(
            int(default["snapshot_gigabytes"]),
            int(reseted["snapshot_gigabytes"]))
        self.assertEqual(
            int(default["share_networks"]), int(reseted["share_networks"]))
        if (utils.is_microversion_supported(SHARE_GROUPS_MICROVERSION) and
                CONF.share.run_share_group_tests):
            self.assertEqual(
                int(default["share_groups"]), int(reseted["share_groups"]))
            self.assertEqual(
                int(default["share_group_snapshots"]),
                int(reseted["share_group_snapshots"]))
        if utils.share_replica_quotas_are_supported():
            self.assertEqual(
                int(default["share_replicas"]), int(reseted["share_replicas"]))
            self.assertEqual(
                int(default["replica_gigabytes"]),
                int(reseted["replica_gigabytes"]))

    def _get_new_replica_quota_values(self, default_quotas, value_to_set):
        new_values = {
            'share_replicas': int(
                default_quotas['share_replicas']) + value_to_set,
            'replica_gigabytes': int(
                default_quotas['replica_gigabytes']) + value_to_set
        }
        return new_values

    @ddt.data(
        ('id', True),
        ('name', False),
    )
    @ddt.unpack
    @decorators.idempotent_id('15e57302-5a14-4be4-8720-95b639c2bfad')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    @utils.skip_if_microversion_not_supported("2.39")
    def test_reset_share_type_quotas(self, share_type_key, is_st_public):
        share_type = self.create_share_type(is_public=is_st_public)
        quota_keys = ['shares', 'snapshots', 'gigabytes', 'snapshot_gigabytes']

        # get default_quotas
        default_quotas = self.client.default_quotas(
            self.tenant_id)['quota_set']

        kwargs = {}

        # check if the replica_gigabytes and share_replicas quotas are
        # supported
        if utils.share_replica_quotas_are_supported():
            kwargs.update(self._get_new_replica_quota_values(
                default_quotas, 5))
            quota_keys.append('share_replicas')
            quota_keys.append('replica_gigabytes')

        # set new project quota
        updated_p_quota = self.update_quotas(
            self.tenant_id,
            shares=int(default_quotas['shares']) + 5,
            snapshots=int(default_quotas['snapshots']) + 5,
            gigabytes=int(default_quotas['gigabytes']) + 5,
            snapshot_gigabytes=int(default_quotas['snapshot_gigabytes']) + 5,
            **kwargs
        )

        if utils.share_replica_quotas_are_supported():
            kwargs.update(self._get_new_replica_quota_values(
                default_quotas, 3))

        # set share type quota for project, don't cleanup, we'll do that below
        self.update_quotas(
            self.tenant_id,
            share_type=share_type[share_type_key],
            shares=int(default_quotas['shares']) + 3,
            snapshots=int(default_quotas['snapshots']) + 3,
            gigabytes=int(default_quotas['gigabytes']) + 3,
            snapshot_gigabytes=int(default_quotas['snapshot_gigabytes']) + 3,
            cleanup=False,
            **kwargs
        )

        # reset share type quotas
        self.client.reset_quotas(
            self.tenant_id, share_type=share_type[share_type_key])

        # verify quotas
        current_p_quota = self.client.show_quotas(self.tenant_id)['quota_set']
        current_st_quota = self.client.show_quotas(
            self.tenant_id, share_type=share_type[share_type_key])['quota_set']
        for key in quota_keys:
            self.assertEqual(updated_p_quota[key], current_p_quota[key])

            # Default share type quotas are current project quotas
            self.assertNotEqual(default_quotas[key], current_st_quota[key])
            self.assertEqual(current_p_quota[key], current_st_quota[key])

    @decorators.idempotent_id('d4bba375-7111-4b93-b6dd-4f0532febc3e')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    def test_unlimited_quota_for_shares(self):
        self.update_quotas(self.tenant_id, shares=-1)

        quotas = self.client.show_quotas(self.tenant_id)['quota_set']

        self.assertEqual(-1, quotas.get('shares'))

    @decorators.idempotent_id('756ffd0e-a476-49af-ac85-9bb4ce5e29b7')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    def test_unlimited_user_quota_for_shares(self):
        self.update_quotas(self.tenant_id, user_id=self.user_id, shares=-1)

        quotas = self.client.show_quotas(
            self.tenant_id, self.user_id)['quota_set']

        self.assertEqual(-1, quotas.get('shares'))

    @decorators.idempotent_id('9779d166-09d3-4745-8acc-2243eadec3ea')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    def test_unlimited_quota_for_snapshots(self):
        self.update_quotas(self.tenant_id, snapshots=-1)

        quotas = self.client.show_quotas(self.tenant_id)['quota_set']

        self.assertEqual(-1, quotas.get('snapshots'))

    @decorators.idempotent_id('245b3bf3-09ef-4b6d-8643-f156bf1bf23c')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    def test_unlimited_user_quota_for_snapshots(self):
        self.update_quotas(self.tenant_id, user_id=self.user_id, snapshots=-1)

        quotas = self.client.show_quotas(
            self.tenant_id, self.user_id)['quota_set']

        self.assertEqual(-1, quotas.get('snapshots'))

    @decorators.idempotent_id('b6a94e87-091a-48dc-9b51-13d81541869c')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    def test_unlimited_quota_for_gigabytes(self):
        self.update_quotas(self.tenant_id, gigabytes=-1)

        quotas = self.client.show_quotas(self.tenant_id)['quota_set']

        self.assertEqual(-1, quotas.get('gigabytes'))

    @decorators.idempotent_id('0d044db4-ae5b-416d-aa51-098afb72cd6c')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    def test_unlimited_quota_for_snapshot_gigabytes(self):
        self.update_quotas(self.tenant_id, snapshot_gigabytes=-1)

        quotas = self.client.show_quotas(self.tenant_id)['quota_set']

        self.assertEqual(-1, quotas.get('snapshot_gigabytes'))

    @decorators.idempotent_id('2de4b7cf-9189-413f-858b-860ecf5fd18b')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    def test_unlimited_user_quota_for_gigabytes(self):
        self.update_quotas(self.tenant_id, user_id=self.user_id, gigabytes=-1)

        quotas = self.client.show_quotas(
            self.tenant_id, self.user_id)['quota_set']

        self.assertEqual(-1, quotas.get('gigabytes'))

    @decorators.idempotent_id('cbe63027-1108-4779-9fd3-22f41f60d6bb')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    def test_unlimited_user_quota_for_snapshot_gigabytes(self):
        self.update_quotas(self.tenant_id,
                           user_id=self.user_id,
                           snapshot_gigabytes=-1)

        quotas = self.client.show_quotas(
            self.tenant_id, self.user_id)['quota_set']

        self.assertEqual(-1, quotas.get('snapshot_gigabytes'))

    @decorators.idempotent_id('e35455f5-92db-4669-ac21-9daf170df248')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    def test_unlimited_quota_for_share_networks(self):
        self.update_quotas(self.tenant_id, share_networks=-1)

        quotas = self.client.show_quotas(self.tenant_id)['quota_set']

        self.assertEqual(-1, quotas.get('share_networks'))

    @decorators.idempotent_id('66b04887-e611-4d4f-a40b-c8b14766b6af')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    def test_unlimited_user_quota_for_share_networks(self):
        self.update_quotas(self.tenant_id,
                           user_id=self.user_id,
                           share_networks=-1)

        quotas = self.client.show_quotas(
            self.tenant_id, self.user_id)['quota_set']

        self.assertEqual(-1, quotas.get('share_networks'))

    @decorators.idempotent_id('7c2cd2d4-4352-4811-9e39-70f56e6297c2')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    @testtools.skipUnless(
        CONF.share.run_share_group_tests, 'Share Group tests disabled.')
    @utils.skip_if_microversion_not_supported(SHARE_GROUPS_MICROVERSION)
    def test_unlimited_quota_for_share_groups(self):
        self.update_quotas(self.tenant_id, share_groups=-1)

        quotas = self.client.show_quotas(self.tenant_id)['quota_set']

        self.assertEqual(-1, quotas.get('share_groups'))

    @decorators.idempotent_id('76d270d5-f314-47cb-9c3f-409f8ff12ce2')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    @testtools.skipUnless(
        CONF.share.run_share_group_tests, 'Share Group tests disabled.')
    @utils.skip_if_microversion_not_supported(SHARE_GROUPS_MICROVERSION)
    def test_unlimited_user_quota_for_share_group_snapshots(self):
        self.update_quotas(self.tenant_id,
                           user_id=self.user_id,
                           share_group_snapshots=-1)

        quotas = self.client.show_quotas(
            self.tenant_id, self.user_id)['quota_set']

        self.assertEqual(-1, quotas.get('share_group_snapshots'))

    @ddt.data("share_replicas", "replica_gigabytes")
    @decorators.idempotent_id('15aa5df5-b2ae-4a3a-acb8-efbbc84581be')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    @utils.skip_if_microversion_not_supported(
        SHARE_REPLICA_QUOTAS_MICROVERSION)
    def test_unlimited_quota_for_replica_quotas(self, quota_key):
        kwargs = {quota_key: -1}
        self.update_quotas(self.tenant_id, **kwargs)

        quotas = self.client.show_quotas(self.tenant_id)['quota_set']

        self.assertEqual(-1, quotas.get(quota_key))

    @ddt.data("share_replicas", "replica_gigabytes")
    @decorators.idempotent_id('84b99731-f748-44fe-a291-162d05da9e25')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    @utils.skip_if_microversion_not_supported(
        SHARE_REPLICA_QUOTAS_MICROVERSION)
    def test_unlimited_user_quota_for_replica_quotas(self, quota_key):
        kwargs = {quota_key: -1}
        self.update_quotas(self.tenant_id, user_id=self.user_id, **kwargs)

        quotas = self.client.show_quotas(
            self.tenant_id, self.user_id)['quota_set']

        self.assertEqual(-1, quotas.get(quota_key))

    @ddt.data(11, -1)
    @decorators.idempotent_id('43f58705-3cad-46bc-816c-41e8fa55dd8d')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    def test_update_user_quotas_bigger_than_project_quota(self, user_quota):
        self.update_quotas(self.tenant_id, shares=10)

        self.update_quotas(self.tenant_id,
                           user_id=self.user_id,
                           force=True,
                           shares=user_quota)

    @ddt.data(11, -1)
    @decorators.idempotent_id('315cb76f-920d-4cb9-ac7d-16be8e95e1b2')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    @utils.skip_if_microversion_not_supported("2.39")
    def test_update_share_type_quotas_bigger_than_project_quota(self, st_q):
        share_type = self.create_share_type()

        self.update_quotas(self.tenant_id, shares=10)

        # no need to cleanup share type quota, share type will be deleted at
        # the end of the test
        self.update_quotas(self.tenant_id,
                           share_type=share_type['name'],
                           force=True,
                           cleanup=False,
                           shares=st_q)

    @decorators.idempotent_id('c95be1eb-6331-4c37-9fac-ed6c36270457')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    @utils.skip_if_microversion_not_supported("2.39")
    def test_set_share_type_quota_bigger_than_users_quota(self):
        share_type = self.create_share_type()

        self.update_quotas(self.tenant_id, force=False, shares=13)

        self.update_quotas(self.tenant_id,
                           user_id=self.user_id,
                           force=True,
                           shares=11)

        # Share type quota does not depend on user's quota, so we should be
        # able to update it. No need for cleanup, since the share type will
        # be deleted when the test completes, cleaning up quotas and usages
        self.update_quotas(self.tenant_id,
                           share_type=share_type['name'],
                           force=False,
                           cleanup=False,
                           shares=12)

    @decorators.idempotent_id('4687eb25-17b3-4995-ace2-62f8bda29c57')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    @utils.skip_if_microversion_not_supported("2.39")
    def test_quotas_usages(self):
        # Create share types
        st_1, st_2 = (self.create_share_type()
                      for i in (1, 2))

        # Set quotas for project, user and both share types
        self.update_quotas(self.tenant_id, shares=3, gigabytes=10)

        self.update_quotas(self.tenant_id,
                           user_id=self.user_id,
                           shares=2,
                           gigabytes=7)

        for st in (st_1['id'], st_2['name']):
            # no need for cleanup, since share types will be deleted at the
            # end of the test
            self.update_quotas(self.tenant_id,
                               share_type=st,
                               shares=2,
                               gigabytes=4,
                               cleanup=False)

        # Create share, 4Gb, st1 - ok
        share_1 = self.create_share(
            size=4, share_type_id=st_1['id'], client=self.client,
            cleanup_in_class=False)

        # Try create shares twice, failing on user and share type quotas
        for size, st_id in ((3, st_1['id']), (4, st_2['id'])):
            self.assertRaises(
                lib_exc.OverLimit,
                self.create_share,
                size=size, share_type_id=st_id, client=self.client,
                cleanup_in_class=False)

        # Create share, 3Gb, st2 - ok
        share_2 = self.create_share(
            size=3, share_type_id=st_2['id'], client=self.client,
            cleanup_in_class=False)

        # Check quota usages
        for g_l, g_use, s_l, s_use, kwargs in (
                (10, 7, 3, 2, {}),
                (7, 7, 2, 2, {'user_id': self.user_id}),
                (4, 4, 2, 1, {'share_type': st_1['id']}),
                (4, 3, 2, 1, {'share_type': st_2['name']})):
            quotas = self.client.detail_quotas(
                tenant_id=self.tenant_id, **kwargs)['quota_set']
            self.assertEqual(0, quotas['gigabytes']['reserved'])
            self.assertEqual(g_l, quotas['gigabytes']['limit'])
            self.assertEqual(g_use, quotas['gigabytes']['in_use'])
            self.assertEqual(0, quotas['shares']['reserved'])
            self.assertEqual(s_l, quotas['shares']['limit'])
            self.assertEqual(s_use, quotas['shares']['in_use'])

        # Delete shares and then check usages
        for share_id in (share_1['id'], share_2['id']):
            self.client.delete_share(share_id)
            self.client.wait_for_resource_deletion(share_id=share_id)
        for kwargs in ({}, {'share_type': st_1['name']},
                       {'user_id': self.user_id}, {'share_type': st_2['id']}):
            quotas = self.client.detail_quotas(
                tenant_id=self.tenant_id, **kwargs)['quota_set']
            for key in ('shares', 'gigabytes'):
                self.assertEqual(0, quotas[key]['reserved'])
                self.assertEqual(0, quotas[key]['in_use'])

    def _check_sg_usages(self, quotas, in_use, limit):
        """Helper method for 'test_share_group_quotas_usages' test."""
        self.assertEqual(0, int(quotas['share_groups']['reserved']))
        self.assertEqual(in_use, int(quotas['share_groups']['in_use']))
        self.assertEqual(limit, int(quotas['share_groups']['limit']))

    def _check_sgs_usages(self, quotas, in_use):
        """Helper method for 'test_share_group_quotas_usages' test."""
        self.assertEqual(0, int(quotas['share_group_snapshots']['reserved']))
        self.assertEqual(
            in_use, int(quotas['share_group_snapshots']['in_use']))
        self.assertEqual(1, int(quotas['share_group_snapshots']['limit']))

    def _check_usages(self, sg_in_use, sgs_in_use):
        """Helper method for 'test_share_group_quotas_usages' test."""
        p_quotas = self.client.detail_quotas(
            tenant_id=self.tenant_id)['quota_set']
        u_quotas = self.client.detail_quotas(
            tenant_id=self.tenant_id, user_id=self.user_id)['quota_set']
        self._check_sg_usages(p_quotas, sg_in_use, 3)
        self._check_sg_usages(u_quotas, sg_in_use, 2)
        self._check_sgs_usages(p_quotas, sgs_in_use)
        self._check_sgs_usages(u_quotas, sgs_in_use)

    @decorators.idempotent_id('fe357398-12d4-4a63-b5ae-0d5091ba3442')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    @testtools.skipUnless(
        CONF.share.run_share_group_tests, 'Share Group tests disabled.')
    @utils.skip_if_microversion_not_supported(SHARE_GROUPS_MICROVERSION)
    @testtools.skipUnless(CONF.share.run_snapshot_tests,
                          "Snapshot tests are disabled.")
    @testtools.skipUnless(
        CONF.share.capability_create_share_from_snapshot_support,
        "Tests for shares from snapshots are disabled.")
    def test_share_group_quotas_usages(self):
        # Set quotas for project (3 SG, 1 SGS) and user (2 SG, 1 SGS)
        self.update_quotas(self.tenant_id,
                           share_groups=3,
                           share_group_snapshots=1)

        self.update_quotas(self.tenant_id,
                           user_id=self.user_id,
                           share_groups=2,
                           share_group_snapshots=1)

        # Check usages, they should be 0s
        self._check_usages(0, 0)

        # Create SG1 and check usages
        share_group1 = self.create_share_group(
            share_group_type_id=self.share_group_type_id,
            share_type_ids=[self.share_type_id],
            cleanup_in_class=False,
            client=self.client)
        self._check_usages(1, 0)

        # Create SGS1 and check usages
        sg_snapshot = self.create_share_group_snapshot_wait_for_active(
            share_group1['id'], cleanup_in_class=False, client=self.client)
        self._check_usages(1, 1)

        # Create SG2 from SGS1 and check usages
        share_group2 = self.create_share_group(
            share_group_type_id=self.share_group_type_id,
            cleanup_in_class=False,
            client=self.client,
            source_share_group_snapshot_id=sg_snapshot['id'])
        self._check_usages(2, 1)

        # Try create SGS2, fail, then check usages
        self.assertRaises(
            lib_exc.OverLimit,
            self.create_share_group,
            share_group_type_id=self.share_group_type_id,
            share_type_ids=[self.share_type_id],
            client=self.client, cleanup_in_class=False)
        self._check_usages(2, 1)

        # Delete SG2 and check usages
        self.client.delete_share_group(share_group2['id'])
        self.client.wait_for_resource_deletion(
            share_group_id=share_group2['id'])
        self._check_usages(1, 1)

        # Delete SGS1 and check usages
        self.client.delete_share_group_snapshot(sg_snapshot['id'])
        self.client.wait_for_resource_deletion(
            share_group_snapshot_id=sg_snapshot['id'])
        self._check_usages(1, 0)

        # Delete SG1 and check usages
        self.client.delete_share_group(share_group1['id'])
        self.client.wait_for_resource_deletion(
            share_group_id=share_group1['id'])
        self._check_usages(0, 0)
